import os
import unittest

import metia


class TestWritter(unittest.TestCase):
    media_dir = os.path.sep.join(
        __file__.split(os.path.sep)[:-1] + ["media_files"]
    )
    original_file = f"{media_dir}/Canon Rock.mp3"
    no_meta_file = f"{media_dir}/blank.mp3"

    def setUp(self):
        os.system(
            f'ffmpeg -hide_banner -loglevel quiet -y -i "{self.original_file}" -map_metadata -1 -map 0 -c copy {self.no_meta_file}'
        )

    def tearDown(self):
        if os.path.isfile(self.no_meta_file):
            os.system(f"rm {self.no_meta_file}")

    def test_get_probe(self):
        self.assertEqual(
            metia.probe.Probe(self.original_file),
            metia.writter.Media(self.original_file).probe(),
        )

    def test_create_from_probe(self):
        self.assertEqual(
            metia.writter.Media(self.original_file),
            metia.writter.Media(metia.probe.Probe(self.original_file)),
        )

    def test_path(self):
        self.assertEqual(
            os.path.realpath(metia.writter.Media(self.original_file).path),
            os.path.realpath(self.original_file),
        )

    def test_get_tags(self):
        self.assertEqual(
            metia.probe.Probe(self.original_file).get_tags(),
            metia.writter.Media(self.original_file).get_tags(),
        )

    def test_tag_io(self):
        new_song = metia.writter.Media(self.no_meta_file)
        new_song.set_tag("title", "Canon Rock")
        new_song.write_song_tags()

        new_probe = metia.probe.Probe(self.original_file)
        self.assertIsInstance(new_probe.get_tags(), dict)
        if isinstance(new_probe.get_tags(), dict):
            self.assertEqual(new_probe.get_tag("title"), "Canon Rock")


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from metia import probe


class TestProbe(unittest.TestCase):
    media_dir = os.path.sep.join(
        __file__.split(os.path.sep)[:-1] + ["media_files"]
    )

    def test_read(self):
        self.assertIsInstance(
            probe.Probe(f"{self.media_dir}/no_audio.mp4"), probe.Probe
        )

    def test_wrong_path(self):
        with self.assertRaises(FileNotFoundError) as exception:
            probe.Probe("asdhfo")

    def test_equal_probe(self):
        self.assertEqual(
            probe.Probe(f"{self.media_dir}/no_audio.mp4"),
            probe.Probe(f"{self.media_dir}/no_audio.mp4"),
        )

    def test_equal_dict(self):
        self.assertEqual(
            probe.Probe(f"{self.media_dir}/no_audio.mp4"),
            probe.Probe(f"{self.media_dir}/no_audio.mp4").dict(),
        )

    def test_empty_audio(self):
        clip = probe.Probe(f"{self.media_dir}/no_audio.mp4")
        self.assertEqual(clip.audio_codec(), {})

    def test_audio_codec(self):
        song = probe.Probe(f"{self.media_dir}/Canon Rock.mp3")
        self.assertEqual(song.audio_codec(), {0: "mp3"})

    def test_video_codec(self):
        clip = probe.Probe(f"{self.media_dir}/no_audio.mp4")
        self.assertEqual(clip.video_codec(), {0: "h264"})

    def test_audio_bitrates(self):
        song = probe.Probe(f"{self.media_dir}/Canon Rock.mp3")
        self.assertEqual(song.audio_bitrates(), {0: 320000})

    def test_video_bitrates(self):
        clip = probe.Probe(f"{self.media_dir}/no_audio.mp4")
        self.assertEqual(clip.video_bitrates(), {0: 27949364})


if __name__ == "__main__":
    unittest.main()



import subprocess
import re
from typing import Iterator
from . import writter


def get_decoders() -> Iterator[str]:
    command = f"{writter.FFMPEG_COMMAND} -hide_banner -decoders"
    output = (
        subprocess.run(command.split(" "), capture_output=True)
        .stdout.decode()
        .strip()
        .split("\n")
    )

    pattern = re.compile(r"\s......\s(\w+)\s*.*")
    for i in output:
        match_result = pattern.match(i)
        if match_result is None:
            continue
        yield match_result.groups()[0]


def get_encoders() -> Iterator[str]:
    command = f"{writter.FFMPEG_COMMAND} -hide_banner -encoders"
    output = (
        subprocess.run(command.split(" "), capture_output=True)
        .stdout.decode()
        .strip()
        .split("\n")
    )

    pattern = re.compile(r"\s......\s(\w+)\s*.*")
    for i in output:
        match_result = pattern.match(i)
        if match_result is None:
            continue
        yield match_result.groups()[0]


if __name__ == "__main__":
    pass



import os
import json
import subprocess
from typing import Dict, Union, Optional


FFPROBE_COMMAND: str = "ffprobe"


class Probe:
    def __init__(self, path: str, encoding: str = "utf-8"):
        """
        Initialize FFprobe object.
        This requires ffprobe command added to your $PATH.
        To specify the path to ffprobe binary, change the value of metia.FFPROBE_COMMAND.
        path: path of video/audio file
        encoding: encoding of command line output. Default is utf-8.
        """
        path = os.path.realpath(os.path.expanduser(path))
        if not os.path.isfile(path):
            raise FileNotFoundError("File not found: {}".format(path))
        self.__path = path
        self.__meta = {}
        self._init(encoding)

    def __hash__(self):
        return hash(str(self))

    def __str__(self):
        return json.dumps(self.__meta, indent=4)

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        if isinstance(other, Probe):
            return hash(self) == hash(other)
        elif isinstance(other, dict):
            return self.__meta == other
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def dict(self):
        """
        Return the dict returned by the json parser.
        """
        return self.__meta.copy()

    def _init(self, encoding="utf8"):
        command = '{} -v quiet -hide_banner -print_format json -show_format -show_streams "{}"'.format(
            FFPROBE_COMMAND, self.__path
        )
        output = subprocess.check_output(command, shell=True)
        self.__meta = json.loads(output.decode(encoding))

    def get_tags(self) -> Optional[Dict[str, str]]:
        if self.__meta.get("format") and self.__meta["format"].get("tags"):
            return self.__meta["format"]["tags"]

    def get_tag(self, key: str) -> Optional[str]:
        if self.__meta.get("format") and self.__meta["format"].get("tags"):
            return self.__meta["format"]["tags"].get(
                key.upper()
            ) or self.__meta["format"]["tags"].get(key.lower())

    @property
    def path(self):
        return os.path.realpath(self.__path)

    def audio_codec(self) -> Dict[int, str]:
        """
        Return a dictionary of audio codecs.
        key: index of stream
        value: codec name
        """
        codecs = {}
        for stream in self.__meta["streams"]:
            if stream["codec_type"] == "audio":
                codecs[stream["index"]] = stream["codec_name"]
        return codecs

    def video_codec(self) -> Dict[int, str]:
        """
        Return a dictionary of video codecs.
        key: index of stream
        value: codec name
        """
        codecs = {}
        for stream in self.__meta["streams"]:
            if stream["codec_type"] == "video":
                codecs[stream["index"]] = stream["codec_name"]
        return codecs

    def audio_bitrates(self) -> Union[Optional[int], Dict[int, str]]:
        """
        Return a dictionary of audio bitrates.
        key: index of stream
        value: bitrate (in bps)
        """
        bitrates = {}
        for stream in self.__meta["streams"]:
            if stream["codec_type"] == "audio":
                try:
                    bitrates[stream["index"]] = int(stream["bit_rate"])
                except KeyError:
                    continue

        if bitrates == {} and self.video_codec() != {}:
            return self.__bitrate()
        return bitrates

    def video_bitrates(self) -> Dict[int, int]:
        """
        Return a dictionary of video bitrates.
        key: index of stream
        value: bitrate (in bps)
        """
        bitrates = {}
        for stream in self.__meta["streams"]:
            if stream["codec_type"] == "video":
                try:
                    bitrates[stream["index"]] = int(stream["bit_rate"])
                except KeyError:
                    continue
        if (
            bitrates == {}
            and self.video_codec() != {}
            and isinstance(self.__bitrate(), int)
        ):
            return {0: self.__bitrate()}
        return bitrates

    def __bitrate(self) -> Optional[int]:
        if (
            self.__meta.get("format") != None
            and self.__meta["format"].get("bit_rate") != None
        ):
            return int(self.__meta["format"]["bit_rate"])

    def video_bitrate_sum(self) -> Optional[int]:
        """
        Return the sum of the bitrates of all video streams.
        """
        if isinstance(self.video_bitrates(), int):
            return int(self.video_bitrates())
        elif isinstance(self.video_bitrates(), dict):
            return sum(value for key, value in dict(self.video_bitrates()))


if __name__ == "__main__":
    pass

del Dict, Union, Optional



import setuptools
import os


if os.path.isfile("README.md"):
    with open("README.md", encoding="utf8") as fin:
        long_description = fin.read()
else:
    long_description = ""

requirements = []
if os.path.isfile("requirements"):
    with open("requirements.txt") as fin:
        requirements.append(i.replace("\n", "") for i in fin.readlines())

setuptools.setup(
    name="metia",
    version="0.3.1",
    author="David Yu",
    author_email="hzjlyz@gmail.com",
    description="A tool to parse and extract audio metadata.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Davidyz/metia",
    project_urls={"Bug Tracker": "https://github.com/Davidyz/metia/issues"},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS",
        "Topic :: Multimedia",
    ],
    entry_points={
        "console_scripts": ["metia-probe = metia.executables:metia_probe"]
    },
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
    install_requires=requirements,
)



import os
import uuid
from typing import Dict, Optional, Union

from . import probe

FFMPEG_COMMAND = "ffmpeg"


class Media:
    __SONG_TAGS = {"album", "artist", "title"}

    def __init__(self, media: Union[probe.Probe, str]):
        if isinstance(media, str):
            if os.path.isfile(media):
                self.__probe: probe.Probe = probe.Probe(media)
            else:
                raise FileNotFoundError(f"{media} is not found.")
        elif isinstance(media, probe.Probe):
            self.__probe: probe.Probe = probe.Probe(media.path)
        elif not isinstance(media, (probe.Probe, str)):
            raise TypeError(
                "media has to be one of the following types: str, probe.Probe."
            )
        self.__cover: Optional[str] = None
        self.__tags: Dict[str, str] = (
            self.get_tags() if not (self.get_tags() is None) else {}
        )
        self.__filename = self.__probe.path.split(os.path.sep)[-1]
        self.__suffix = self.__filename.split(".")[-1]

    def __eq__(self, other) -> bool:
        return (
            isinstance(other, (Media, probe.Probe)) and self.path == other.path
        )

    @property
    def path(self) -> str:
        return self.__probe.path

    def probe(self) -> probe.Probe:
        return self.__probe

    def find_song_cover(self, path: Optional[str] = None) -> bool:
        """
        Try to set the cover image.
        If an argument is not provided, it tries to find cover.jpg from the same directory.
        Return True if image is set.
        """
        if isinstance(self.__cover, str) and os.path.isfile(self.__cover):
            # cover is already found
            return True
        elif any(
            self.__probe.video_codec()[i] == "mpeg"
            for i in self.__probe.video_codec().keys()
        ):
            # the file already has a cover art
            return True
        elif (
            isinstance(path, str)
            and path.endswith(".jpg")
            and os.path.isfile(path)
        ):
            self.__cover = path
            return True
        cover_names = ["cover.jpg", "Cover.jpg"]
        song_dir = os.path.sep.join(self.__probe.path.split(os.path.sep)[:-1])
        if path is None:
            # trying to find the cover from the same directory
            for name in cover_names:
                existing_cover_path = os.path.join(song_dir, name)
                if os.path.isfile(existing_cover_path):
                    self.__cover = existing_cover_path
                    return True
            return False
        elif not os.path.isfile(path):
            raise FileNotFoundError(f"{path} is not a file.")

        path = os.path.expanduser(str(path))

        try:
            probe.Probe(path)
            os.system(f'cp "{path}" "{song_dir}"')
        except Exception:
            return False

        return isinstance(self.__cover, str)

    def get_tags(self) -> Dict[str, str]:
        return self.__probe.get_tags() or {}

    def get_tag(self, key: str) -> Optional[str]:
        return self.__probe.get_tag(key)

    def set_tag(self, key: str, value: str, encoding="utf8") -> None:
        self.__tags[key.upper() if key in self.__SONG_TAGS else key] = value

    def set_song_cover(self):
        """
        Embed the album art into the media.
        This requires a self.__cover to be set to an image. If not, a FileNotFoundError will be thrown.
        """
        if not self.find_song_cover():
            raise FileNotFoundError(
                "Failed to find a cover art in the directory of the media."
            )
        elif any(
            self.__probe.video_codec()[i] == "mjpeg"
            for i in self.__probe.video_codec().keys()
        ):
            # the file already has a cover art
            return
        random_name = str(uuid.uuid4()) + "." + self.__suffix
        command = f'{FFMPEG_COMMAND} -i "{self.__probe.path}" -i "{self.__cover}" -loglevel 0 -hide_banner -map 0:0 -map 1:0 -c copy -metadata:s:v title="Album cover" -metadata:s:v comment="Cover (front)" -disposition:v attached_pic /tmp/{random_name}'
        if os.system(command) == 0:
            os.system(f'mv /tmp/{random_name} "{self.__probe.path}"')

    def write_song_tags(self):
        command = f'{FFMPEG_COMMAND} -hide_banner -loglevel 0 -i "{self.path}" -map 0 -c copy'
        for key in self.__SONG_TAGS:
            value = self.get_tags().get(key.upper())
            if isinstance(value, str):
                command += " -metadata {key}={value} ".format(
                    key=key, value=value.replace(" ", "\\ ")
                )
        random_name = str(uuid.uuid4()) + "." + self.__suffix
        command += f" /tmp/{random_name}"
        os.system(command)
        os.system(f'mv /tmp/{random_name} "{self.path}"')


if __name__ == "__main__":
    pass



#!/usr/bin/env python3
import collections
import os
import sys
from typing import Dict, Iterable, List, Union
from metia.probe import Probe

try:
    from collections import Iterable
except (AttributeError, ImportError):
    from collections.abc import Iterable


__INDENT_STRING = "  "


def pprint_list(info: List, indentation: int = 0) -> None:
    indent_str = indentation * __INDENT_STRING
    for i in info:
        if isinstance(i, list):
            pprint_list(i, indentation + 1)
        elif isinstance(i, dict):
            pprint_dict(i, indentation)
        else:
            print(f"{indent_str}{i}")


def pprint_dict(
    info: Dict[str, Union[Dict, str]], indentation: int = 0
) -> None:
    """
    Print the metadata of the media in a nice format.
    """
    indent_str = __INDENT_STRING * indentation
    for key in sorted(info.keys()):
        value = info[key]
        if isinstance(value, dict):
            print(f"{indent_str}{key}:")
            pprint_dict(value, indentation + 1)
        elif isinstance(value, list):
            print(f"{indent_str}{key}:")
            pprint_list(sorted(value), indentation + 1)
        else:
            print(f"{indent_str}{key}: {value}")


def print_media(media: Probe, key) -> int:
    """
    Driver function for pprint()
    """
    print(media.path)
    print(len(media.path) * "=")
    if key is None:
        pprint_dict(media.dict())
    else:
        data = find_val(media.dict(), key)
        if data != None:
            print(f"{key}: ", end="")
            if isinstance(data, dict):
                print()
                pprint_dict(data, 1)
            elif isinstance(data, list):
                print()
                pprint_list(data, 1)
            else:
                print(data)
        else:
            print(f"The key {key} is not found in the media.")
            return 1
    return 0


def find_val(original: Dict, key: str):
    q = collections.deque((original,))
    while q:
        candidate = q.popleft()
        for i in candidate.keys():
            if i == key:
                return candidate[i]
            if isinstance(candidate[i], dict):
                q.append(candidate[i])
            elif isinstance(candidate[i], Iterable) and not isinstance(
                candidate[i], str
            ):
                result = find_val_from_iter(candidate[i], key)
                if result is None:
                    continue
                return result
    return None


def find_val_from_iter(iterable: Iterable, key: str):
    q = collections.deque()
    q.append(iterable)
    while q:
        candidate = q.popleft()
        for i in candidate:
            if isinstance(i, dict):
                return find_val(i, key)
            elif isinstance(i, Iterable) and not isinstance(i, str):
                q.append(i)
    return None


def metia_probe():
    print("A tool to visualize the metadata of a media file.")
    path, key = None, None
    if len(sys.argv) == 1:
        print("Usage: media-probe [PATH_TO_MEDIA]")
        sys.exit(1)
    elif len(sys.argv) == 2:
        path = sys.argv[1]

    elif len(sys.argv) == 3:
        path, key = sys.argv[1:]
    else:
        print("Usage: media-probe [PATH_TO_MEDIA] [key]")
        sys.exit(1)
    if not os.path.isfile(path):
        print(f"Not a valid path: {path}")
        sys.exit(1)

    meta = Probe(path)
    sys.exit(print_media(meta, key))
